#define F_CPU 16000000L
#include <util/delay.h>
#include <limits.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <avr/eeprom.h>
#include "Lib_Generic.h"

#define SLAVEID	1

#define BTN1 PC0
#define BTN2 PC1
#define BTN3 PC2
volatile int BTN1_LAST=0, BTN2_LAST=0, BTN3_LAST=0;

int previous_rand = 0;
int mistakes_left = 7;
int letters_count = 10;
int selected_level = 0;
char points = '3';
char curr_text[15] = {0};
char guess_text[15] = {0};
char wrong_guesses[8] = {0};
char selected_char = 'A';
char level_names[3][7] = {"EASY","MEDIUM", "HARD"};
char easy_words[10][20] = {"CAT", "PIG", "FAT", "RAT", "BAG", "PET", "BAT", "BIG", "DUCK", "MICE"};
char medium_words[10][20] = {"MOUSE", "TULIP", "RAINBOW", "TRAIN", "DETOX", "POLICE", "ELECTRO", "MAYDAY", "PRICE", "MUTEX"};
char hard_words[10][20] = {"BUCHAREST", "ROBOCOPS", "STRANGER", "SUNSHINE", "THANKSGIVING", "POYITECHNIC", "SOLIDARITY", "HAPPYNESS", "BUILDING", "RASPBERRY"};
volatile int is_char_selected = 0;
volatile int is_Game_over = 0;
volatile int is_level_selected = 0;

ISR(PCINT2_vect)
{
	
	cli();
	int state_curr_btn1 = (PINC & (1 << BTN1));
	int state_curr_btn2 = (PINC & (1 << BTN2));
	int state_curr_btn3 = (PINC & (1 << BTN3));
	
	if(is_level_selected == 0)
	{
		if (BTN1_LAST != state_curr_btn1)
		{
			if (state_curr_btn1 && is_level_selected == 0)
			{
				selected_level++;
				if(selected_level > 2)
				selected_level = 0;
				_delay_ms(100);
			}
			BTN1_LAST = state_curr_btn1;
			sei();
		}
		if (BTN2_LAST != state_curr_btn2)
		{
			if (state_curr_btn2 && is_level_selected == 0)
			{
				selected_level--;
				if(selected_level < 0)
				selected_level = 2;
				_delay_ms(100);
			}
			BTN2_LAST = state_curr_btn2;
			sei();
		}
		if (BTN3_LAST != state_curr_btn3)
		{
			if (state_curr_btn3 && is_level_selected == 0)
			{
				is_level_selected = 1;
				_delay_ms(100);
			}
			BTN3_LAST = state_curr_btn3;
			sei();
		}
	}
	else
	if(is_Game_over == 0)
	{
		if (BTN1_LAST != state_curr_btn1)
		{
			if (state_curr_btn1 && !is_char_selected)
			{
				selected_char++;
				if(selected_char > 'Z')
				selected_char = 'A';
				_delay_ms(100);
			}
			BTN1_LAST = state_curr_btn1;
			sei();
		}
		if (BTN2_LAST != state_curr_btn2)
		{
			if (state_curr_btn2 && !is_char_selected)
			{
				selected_char--;
				if(selected_char < 'A')
				selected_char = 'Z';
				_delay_ms(100);
			}
			
			BTN2_LAST = state_curr_btn2;
			sei();
		}
		if (BTN3_LAST != state_curr_btn3)
		{
			if (state_curr_btn3)
			{
				is_char_selected = 1;
				_delay_ms(100);
			}
			BTN3_LAST = state_curr_btn3;
			sei();
		}
	}
}

void DrawHangingDevice()
{
	drawRect(20, 10, 2, 80, GREEN, SLAVEID);
	drawRect(20, 10, 20, 2, GREEN, SLAVEID);
	drawRect(40, 10, 2, 10, GREEN, SLAVEID);
	drawRect(5, 90, 30, 2, GREEN, SLAVEID);
}

void UpdateText()
{
	int pos = 5;

	fillRect(0, 92, 160, 128-90, BLACK, SLAVEID);
	for (int i=0; i<letters_count; i++)
	{
		drawChar(pos, 100, curr_text[i], BLUE, BLACK, 2, SLAVEID);
		pos += 12;
	}
}

void DrawHangman()
{
	if (mistakes_left < 7)
	drawCircle(40, 30, 10, RED, SLAVEID);
	else
	drawCircle(40, 30, 10, BLACK, SLAVEID);

	if (mistakes_left < 6)
	drawRect(40, 40, 2, 10, RED, SLAVEID);
	else
	drawRect(40, 40, 2, 10, BLACK, SLAVEID);

	if (mistakes_left < 5)
	drawRect(40, 50, 2, 20, RED, SLAVEID);
	else
	drawRect(40, 50, 2, 20, BLACK, SLAVEID);

	if (mistakes_left < 4)
	drawLine(28, 55, 40, 50, RED, SLAVEID);
	else
	drawLine(28, 55, 40, 50, BLACK, SLAVEID);

	if (mistakes_left < 3)
	drawLine(40, 50, 52, 55, RED, SLAVEID);
	else
	drawLine(40, 50, 52, 55, BLACK, SLAVEID);

	if (mistakes_left < 2)
	drawLine(25, 75, 40, 70, RED, SLAVEID);
	else
	drawLine(25, 75, 40, 70, BLACK, SLAVEID);

	if (mistakes_left < 1)
	drawLine(40, 70, 55, 75, RED, SLAVEID);
	else
	drawLine(40, 70, 55, 75, BLACK, SLAVEID);
}

void DrawPoints()
{
	int pos = 60;
	char level[10];
	strcpy(level, level_names[selected_level]);

	for (int i=0; i<strlen(level); i++)
	{
		drawChar(pos, 10, level[i], WHITE, BLACK, 2, SLAVEID);
		pos += 12;
	}

	pos += 12;
	drawChar(pos, 10, points, GREEN, BLACK, 2, SLAVEID);
}

void DrawLevel()
{
	int pos = 35;
	char levelname[7], selecter_description[30];
	
	strcpy(selecter_description, "Please select level:");
	fillScreen(BLACK, SLAVEID);
	for (int i=0; i<strlen(selecter_description); i++)
	{
		drawChar(pos, 30, selecter_description[i], BLUE, BLACK, 1.8, SLAVEID);
		pos += 5;
	}

	pos = 50;
	strcpy(levelname, level_names[selected_level]);
	for (int i=0; i<strlen(levelname); i++)
	{
		drawChar(pos, 70, levelname[i], BLUE, BLACK, 2, SLAVEID);
		pos += 12;
	}
	
}

void GameDescription()
{
	is_level_selected = 0;
	is_Game_over = 0;
	fillScreen(BLACK, SLAVEID);
	int pos = 40;
	char game_name[10], year[5];
	strcpy(game_name, "HANGMAN");
	strcpy(year, "2017");

	for (int i=0; i<strlen(game_name); i++)
	{
		drawChar(pos, 30, game_name[i], RED, BLACK, 2, SLAVEID);
		pos += 12;
	}

	pos = 60;
	for (int i=0; i<strlen(year); i++)
	{
		drawChar(pos, 50, year[i], RED, BLACK, 2, SLAVEID);
		pos += 12;
	}
	
	_delay_ms(2000);
}

void ChooseWord()
{

	int new_rand_index = random()%10;
	
	while (new_rand_index == previous_rand)
	new_rand_index= random()%10;

	previous_rand = new_rand_index;

	if(selected_level == 0)
	strcpy(guess_text, easy_words[new_rand_index]);
	if(selected_level == 1)
	strcpy(guess_text, medium_words[new_rand_index]);
	if(selected_level == 2)
	strcpy(guess_text, hard_words[new_rand_index]);
	
	letters_count = strlen(guess_text);

	memset(curr_text+1, '_', letters_count-2);

	curr_text[0] = guess_text[0];
	curr_text[strlen(guess_text)-1] = guess_text[strlen(guess_text)-1];

	for (int i=0; i<letters_count; i++)
	{
		if (guess_text[i] == guess_text[0])
		{
			curr_text[i] = guess_text[0];
		}
		else if (guess_text[i] == guess_text[strlen(guess_text)-1])
		{
			curr_text[i] = guess_text[strlen(guess_text)-1];
		}
	}
}

void Updateselected_char()
{
	drawChar(80, 60, selected_char, MAGENTA, BLACK, 3, SLAVEID);
}

void UpdateWrongGuesses()
{
	int pos = 60;

	for (int i=0; i<strlen(wrong_guesses); i++)
	{
		drawChar(pos, 30, wrong_guesses[i], BLUE, BLACK, 2, SLAVEID);
		pos += 12;
	}
}

void GameOver()
{
	int pos = 20;
	char *text = "YOU LOSE!:(";
	mistakes_left = 7;

	points--;
	if(points == '0'){
		text = "YOU HANG!";
	}
	
	fillScreen(BLACK, SLAVEID);
	is_Game_over = 1;
	for (int i=0; i<strlen(text); i++)
	{
		drawChar(pos, 30, text[i], RED, BLACK, 2, SLAVEID);
		pos += 12;
	}
	_delay_ms(2000);

	if(points != '0'){
		fillScreen(BLACK, SLAVEID);
		DrawHangingDevice();
		DrawPoints();
		DrawHangman();
		ChooseWord();
		UpdateText();
		is_Game_over = 0;
		strcpy(wrong_guesses, "");
		selected_char = 'A';
	}
}

void GameWin()
{
	int pos = 20;
	char *text = "YOU WIN! :)";
	fillScreen(BLACK, SLAVEID);
	mistakes_left = 7;

	points++;

	if(points == '5') {
		text = "YOU WINNER!";
		if(selected_level != 2)
		selected_level++;
	}

	if(points == '7') {
		text = "BIG WINNER!";
		if(selected_level != 2)
		selected_level++;
	}

	if(points == ':') {
		text = "THE MASTER!";
	}

	for (int i=0; i<strlen(text); i++)
	{
		drawChar(pos, 30, text[i], GREEN, BLACK, 2, SLAVEID);
		pos += 12;
	}

	_delay_ms(2000);

	if(points != ':')
	{
		fillScreen(BLACK, SLAVEID);
		DrawHangingDevice();
		DrawPoints();
		DrawHangman();
		ChooseWord();
		UpdateText();
		is_Game_over = 0;
		strcpy(wrong_guesses, "");
		selected_char = 'A';
	}
}

int main()
{
	int good_guess;
	sei();
	initR();
	setRotation(3, SLAVEID);
	fillScreen(BLACK, SLAVEID);

	_delay_ms(200);
	DDRD = 0xFF;
	PCICR |= (1 << PCIE2);//enable PCI2
	PCMSK2 |= (1 << PCINT16) | (1 << PCINT17) | (1 << PCINT18) | (1 << PCINT19) | (1 << PCINT20) | (1 << PCINT21);

	/* buttons input and pullup enabled */
	DDRC &= ~(1 << BTN1);
	DDRC &= ~(1 << BTN2);
	DDRC &= ~(1 << BTN3);

	PORTC |= (1 << BTN1);
	PORTC |= (1 << BTN2);
	PORTC |= (1 << BTN3);
	
	srand(PIND^PINA);
	
	_delay_ms(200);
	
	GameDescription();

	RESTART:
	
	is_level_selected = 0;
	int old_set = selected_level;
	DrawLevel();

	while (1)
	{
		if(old_set != selected_level)
		{
			DrawLevel();
			old_set = selected_level;
		}
		if (is_level_selected == 1)
		{
			break;
		}
	}

	is_Game_over = 0;
	is_level_selected = 1;
	points = '3';
	mistakes_left = 7;
	selected_char = 'A';
	strcpy(wrong_guesses, "");

	fillScreen(BLACK, SLAVEID);
	DrawHangingDevice();
	DrawPoints();
	DrawHangman();
	ChooseWord();
	UpdateText();

	while(1)
	{
		_delay_ms(100);
		Updateselected_char();

		if (is_char_selected)
		{
			is_char_selected = 0;
			good_guess = 0;

			for (int i=0; i<letters_count; i++)
			{
				if (guess_text[i] == selected_char)
				{
					curr_text[i] = selected_char;
					good_guess = 1;
				}
			}
			if (good_guess)
			{
				UpdateText();
				if (strchr(curr_text, '_') == NULL)
				{
					GameWin();
					if (points == ':')
					goto RESTART;
				}
			}
			else
			{
				mistakes_left--;
				strncat(wrong_guesses, &selected_char, 1);
				DrawHangman();
				UpdateWrongGuesses();
				if (mistakes_left == 0)
				{
					GameOver();
					if (points == '0')
					goto RESTART;
				}
			}
		}
	}
}